package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.FilterProcessEntity;
import com.gic.spark.entity.bean.TrdEcuBrandBean;
import com.gic.spark.entity.bean.TrdEcuSalesBeanBase;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.util.CommonUtil;
import com.google.common.base.Joiner;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @description: 平均折扣率
 * @author: wangxk
 * @date: 2020/5/7
 */
public class TagAverageDiscountFactorFilter extends AbstractTagConsumFilter {

    private static TagAverageDiscountFactorFilter instance;

    public static TagAverageDiscountFactorFilter getInstance() {
        if (null == instance) {
            instance = new TagAverageDiscountFactorFilter();
        }
        return instance;
    }

    private TagAverageDiscountFactorFilter() {
    }


    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveBrandLabel);
        result.add(dataSourceHiveSalesLabel);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) request;
        JavaRDD<TrdEcuSalesBeanBase> salesLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuSalesBeanBase.class).javaRDD();
        JavaRDD<TrdEcuBrandBean> brandLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandBean.class).javaRDD();

        JavaRDD<Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>>> labelRDD = salesLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .leftOuterJoin(brandLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data)).groupByKey())
                .map(data -> data._2());

        JavaRDD<TrdEcuSalesBeanBase> consumeRDD = statisticsTypeHandle(labelRDD, consumeRequest);

        int configStatus = CommonUtil.getConfigStatus(enterpriseId);
        JavaRDD<Long> ecuRdd = consumeRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .reduceByKey((x, y) -> {
                    x.setReceive_amt(x.getReceive_amt() + y.getReceive_amt());
                    x.setPay_amt(x.getPay_amt() + y.getPay_amt());
                    x.setTotal_amt(x.getTotal_amt() + y.getTotal_amt());
                    return x;
                })
                .mapPartitions(data -> {
                    List<Long> result = new ArrayList();
                    while (data.hasNext()) {
                        Tuple2<Long, TrdEcuSalesBeanBase> tp2 = data.next();
                        double avgDiscountRate = 1 == configStatus ? CommonUtil.isEmptyDouble2double(tp2._2().getPay_amt()) / CommonUtil.isEmptyDouble2double(tp2._2().getTotal_amt())
                                : CommonUtil.isEmptyDouble2double(tp2._2().getReceive_amt()) / CommonUtil.isEmptyDouble2double(tp2._2().getTotal_amt());
                        switch (consumeRequest.getNumberType()) {
                            case gt:
                                if (avgDiscountRate > consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case gte:
                                if (avgDiscountRate >= consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lt:
                                if (avgDiscountRate < consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lte:
                                if (avgDiscountRate <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case eq:
                                if (avgDiscountRate == consumeRequest.getEqualNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case between:
                                if (avgDiscountRate >= consumeRequest.getBeginNum()
                                        && avgDiscountRate <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                            default:
                                break;
                        }
                    }
                    return result.iterator();
                });
        return ecuRdd;
    }

    @Override
    public JavaPairRDD<Long, String> filterValidMember(Integer enterpriseId, List<FilterProcessEntity> processEntityList) {
        JavaRDD<ConsumeStatisticEntity> entityJavaRDD = getConsumeEntity(enterpriseId);
        int configStatus = CommonUtil.getConfigStatus(enterpriseId);

        JavaPairRDD<Long, String> result = entityJavaRDD.mapToPair(data -> {
            Set<String> groupIds = new HashSet<>();
            for (FilterProcessEntity entity : processEntityList) {
                TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) entity.getRequest();

                List<TrdEcuSalesBeanBase> salesBeanList = filterSalesBean(data, entity.getRequest());
                if (salesBeanList.size() == 0) {
                    continue;
                }

                double receiveAmount = 0;
                double payAmount = 0;
                double totalAmount = 0;
                for (TrdEcuSalesBeanBase beanBase : salesBeanList) {
                    receiveAmount += beanBase.getReceive_amt();
                    payAmount += beanBase.getPay_amt();
                    totalAmount += beanBase.getTotal_amt();
                }

                double avgDiscountRate = 1 == configStatus ? payAmount / totalAmount
                        : receiveAmount / totalAmount;
                String tagGroupId = entity.getTagGroupId();
                handleValueCompare(groupIds, consumeRequest, tagGroupId, avgDiscountRate);
            }

            return Tuple2.apply(data.ecuId, groupIds.size() == 0 ? null : Joiner.on(" ").join(groupIds));
        }).filter(data -> data._2() != null);
        return result;
    }
}
