package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.FilterProcessEntity;
import com.gic.spark.entity.bean.TrdEcuBrandBean;
import com.gic.spark.entity.bean.TrdEcuSalesBeanBase;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.entity.request.TagConsumeRequest;
import com.google.common.base.Joiner;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @description: 消费休眠天数
 * @author: wangxk
 * @date: 2020/5/8
 */
public class TagConsumptionSleepDaysFilter extends AbstractTagConsumFilter {

    private static TagConsumptionSleepDaysFilter instance;

    public static TagConsumptionSleepDaysFilter getInstance() {
        if (null == instance) {
            instance = new TagConsumptionSleepDaysFilter();
        }
        return instance;
    }

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveBrandLabel);
        result.add(dataSourceHiveSalesLabel);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeRequest consumeRequest = (TagConsumeRequest) request;
        JavaRDD<TrdEcuSalesBeanBase> salesLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuSalesBeanBase.class).javaRDD();
        JavaRDD<TrdEcuBrandBean> brandLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandBean.class).javaRDD();

        JavaRDD<Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>>> labelRDD = salesLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .leftOuterJoin(brandLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data)).groupByKey())
                .map(data -> data._2());
        JavaRDD<TrdEcuSalesBeanBase> consumeRDD = statisticsTypeHandle(labelRDD, consumeRequest);

        JavaRDD<Long> ecuRdd = consumeRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data.getSleep_days()))
                .mapPartitionsToPair(data -> {
                    List<Tuple2<Long, Integer>> result = new ArrayList();
                    while (data.hasNext()) {
                        Tuple2<Long, Integer> tp2 = data.next();
                        switch (consumeRequest.getNumberType()) {
                            case gt:
                                if (tp2._2() > consumeRequest.getBeginNum()) {
                                    result.add(tp2);
                                }
                                break;
                            case gte:
                                if (tp2._2() >= consumeRequest.getBeginNum()) {
                                    result.add(tp2);
                                }
                                break;
                            case lt:
                                if (tp2._2() < consumeRequest.getEndNum()) {
                                    result.add(tp2);
                                }
                                break;
                            case lte:
                                if (tp2._2() <= consumeRequest.getEndNum()) {
                                    result.add(tp2);
                                }
                                break;
                            case eq:
                                if (tp2._2() == consumeRequest.getEqualNum()) {
                                    result.add(tp2);
                                }
                                break;
                            case between:
                                if (tp2._2() >= consumeRequest.getBeginNum()
                                        && tp2._2() <= consumeRequest.getEndNum()) {
                                    result.add(tp2);
                                }
                            default:
                                break;
                        }
                    }
                    return result.iterator();
                })
                .reduceByKey((x, y) -> x)
                .map(Tuple2::_1);
        return ecuRdd;
    }

    @Override
    public JavaPairRDD<Long, String> filterValidMember(Integer enterpriseId, List<FilterProcessEntity> processEntityList) {
        JavaRDD<ConsumeStatisticEntity> entityJavaRDD = getConsumeEntity(enterpriseId);
        JavaPairRDD<Long, String> rdd = entityJavaRDD.mapToPair(data -> {
            Set<String> groupIds = new HashSet<>();

            for (FilterProcessEntity entity : processEntityList) {
                TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) entity.getRequest();
                List<TrdEcuSalesBeanBase> salseBeanList = filterSalesBean(data, entity.getRequest());

                if (salseBeanList.size() == 0) {
                    continue;
                }

                String tagGroupId = entity.getTagGroupId();

                for (TrdEcuSalesBeanBase beanBase : salseBeanList) {
                    Integer sleepDays = beanBase.getSleep_days();
                    handleValueCompare(groupIds, consumeRequest, tagGroupId, sleepDays);
                }


            }
            return Tuple2.apply(data.ecuId, groupIds.size() == 0 ? null : Joiner.on(" ").join(groupIds));
        }).filter(data -> data._2() != null);
        return rdd;
    }
}
