package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.FilterProcessEntity;
import com.gic.spark.entity.bean.TrdEcuBrandBean;
import com.gic.spark.entity.bean.TrdEcuSalesBeanBase;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.entity.request.TagConsumeRequest;
import com.gic.spark.util.CommonUtil;
import com.google.common.base.Joiner;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @description: 客单价
 * @author: wangxk
 * @date: 2020/5/7
 */
public class TagPerCustomerTransactionFilter extends AbstractTagConsumFilter {

    private static TagPerCustomerTransactionFilter instance;

    public static TagPerCustomerTransactionFilter getInstance() {
        if (null == instance) {
            instance = new TagPerCustomerTransactionFilter();
        }
        return instance;
    }

    private TagPerCustomerTransactionFilter() {
    }

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveBrandLabel);
        result.add(dataSourceHiveSalesLabel);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeRequest consumeRequest = (TagConsumeRequest) request;
        JavaRDD<TrdEcuSalesBeanBase> salesLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuSalesBeanBase.class).javaRDD();
        JavaRDD<TrdEcuBrandBean> brandLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandBean.class).javaRDD();

        JavaRDD<Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>>> labelRDD = salesLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .leftOuterJoin(brandLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data)).groupByKey())
                .map(data -> data._2());
        JavaRDD<TrdEcuSalesBeanBase> consumeRDD = statisticsTypeHandle(labelRDD, consumeRequest);
        int configStatus = CommonUtil.getConfigStatus(enterpriseId);
        JavaRDD<Long> ecuRdd = consumeRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .reduceByKey((x, y) -> {
                    x.setReceive_amt(x.getReceive_amt() + y.getReceive_amt());
                    x.setPay_amt(x.getPay_amt() + y.getPay_amt());
                    x.setOrder_times(x.getOrder_times() + y.getOrder_times());
                    return x;
                })
                .mapPartitions(data -> {
                    List<Long> result = new ArrayList();
                    while (data.hasNext()) {
                        Tuple2<Long, TrdEcuSalesBeanBase> tp2 = data.next();
                        double CusSinglePiece = 1 == configStatus ? CommonUtil.isEmptyDouble2double(tp2._2().getPay_amt()) / CommonUtil.isEmptyInteger2int(tp2._2().getSeff_order_cnt())
                                : CommonUtil.isEmptyDouble2double(tp2._2().getReceive_amt()) / CommonUtil.isEmptyInteger2int(tp2._2().getSeff_order_cnt());
                        switch (consumeRequest.getNumberType()) {
                            case gt:
                                if (CusSinglePiece > consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case gte:
                                if (CusSinglePiece >= consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lt:
                                if (CusSinglePiece < consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lte:
                                if (CusSinglePiece <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case eq:
                                if (CusSinglePiece == consumeRequest.getEqualNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case between:
                                if (CusSinglePiece >= consumeRequest.getBeginNum()
                                        && CusSinglePiece <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                            default:
                                break;
                        }
                    }
                    return result.iterator();
                });
        return ecuRdd;
    }

    @Override
    public JavaPairRDD<Long, String> filterValidMember(Integer enterpriseId, List<FilterProcessEntity> processEntityList) {
        JavaRDD<ConsumeStatisticEntity> entityJavaRDD = getConsumeEntity(enterpriseId);
        int configStatus = CommonUtil.getConfigStatus(enterpriseId);
        return entityJavaRDD.mapToPair(data -> {
            Set<String> groupIds = new HashSet<>();

            for (FilterProcessEntity entity : processEntityList) {
                TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) entity.getRequest();
                List<TrdEcuSalesBeanBase> salseBeanList = filterSalesBean(data, entity.getRequest());

                if (salseBeanList.size() == 0) {
                    continue;
                }

                String tagGroupId = entity.getTagGroupId();

                double receiveAmount = 0;
                double payAmount = 0;
                int orderTimes = 0;

                for (TrdEcuSalesBeanBase beanBase : salseBeanList) {
                    receiveAmount += beanBase.getReceive_amt();
                    payAmount += beanBase.getPay_amt();
                    orderTimes += beanBase.getOrder_times();
                }

                double cusSinglePiece = 1 == configStatus ? payAmount / orderTimes
                        : receiveAmount / orderTimes;

                handleValueCompare(groupIds, consumeRequest, tagGroupId, cusSinglePiece);
            }
            return Tuple2.apply(data.ecuId, groupIds.size() == 0 ? null : Joiner.on(" ").join(groupIds));
        }).filter(data -> data._2() != null);
    }
}
